<?php
namespace Teknisa\SM\Controller;

use Teknisa\SM\Controller\SMCrud;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DataSource\DataSet;
use Teknisa\SM\Service\Sm00006002VariacaoTipoDeMedidaService as variacaoTpMedidaService;

class Sm00006002VariacaoTipoDeMedidaController extends SMCrud
{
    protected $dataSourceName = 'fct_variacaotipomedida_fct_tipomedidaserv';
    protected $variacaoTpMedidaService;
     
    public function __construct ($crudService, $environment, variacaoTpMedidaService $variacaoTpMedidaService) {
        parent::__construct($crudService, $environment);
        $this->variacaoTpMedidaService = $variacaoTpMedidaService;
    }
      
    public function save(Request $request, Response $response) {
        $request->setParameter('dbname', $this->dataSourceName);
        parent::save($request, $response);
	    $response->addDataSet(new DataSet('columnsToReturn', $this->crudService->getColumnsToReturn()));
    }
    
    /**
     * A função "validaTipoMedidaVariavel" verifica se o tipo de medida variável
     * ja está criado para a faixa existente.
     */
    
    public function validaTipoMedidaVariavel(Request\Row $request, Response $response) {
    
        $row = $request->getRow();
        
        $cdTpMedidaServ = $row['CDTPMEDIDASERV']; 
        $CdtpMedidaVariavel = $row['CDTPMEDIDAVARIAVEL']; 
        $nrVaricacaoTpMedida = $row['NRVARIACAOTPMEDIDA'];
        
        $ValidaVariacaoTpMed = $this->variacaoTpMedidaService->validaTipoMedidaVariavel($cdTpMedidaServ, $CdtpMedidaVariavel, $nrVaricacaoTpMedida);
        
        $dtsVariacaoTpMed = new DataSet('VALIDACAO', $ValidaVariacaoTpMed);
        
        $response->addDataSet($dtsVariacaoTpMed);
    
    }   
    
      /**
     * A função "validaExsiteMedidaVariavel" verifica se existem medidas variaveis 
     * para o tipo de medida atual.
     */
    
    public function validaExsiteMedidaVariavel(Request\Row $request, Response $response) {
    
        $row = $request->getRow();
        
        $cdTpMedidaServ = $row['CDTPMEDIDASERV']; 
        
        $ValidaExisteMedVariavel = $this->variacaoTpMedidaService->validaExsiteMedidaVariavel($cdTpMedidaServ);
        
        $dtsValExisteMedVariavel = new DataSet('MEDIDAVARIAVEL', array('VALIDA'=>$ValidaExisteMedVariavel));
        
        $response->addDataSet($dtsValExisteMedVariavel);
    
    } 
    
      /**
     * A função "validaExsiteVariacao" verifica se o tipo de medida variavel esá sendo 
     * utilizado.
     */
    
    public function validaExsiteVariacao(Request\Row $request, Response $response) {
    
        $row = $request->getRow();
        
        $CdtpMedidaVariavel = $row['CDTPMEDIDAVARIAVEL']; 
        
        $ValidaExsiteVariacao = $this->variacaoTpMedidaService->validaExsiteVariacao($CdtpMedidaVariavel);
        
        $dtsValExsiteVariacao = new DataSet('VARIACAO', array('VALIDA'=>$ValidaExsiteVariacao));
        
        $response->addDataSet($dtsValExsiteVariacao);
    
    } 
    
    public function delete(Request $request, Response $response) {
    
        $request->setParameter('dbname', $this->dataSourceName);
        parent::delete($request, $response);
    
    }
    
    public function find(Request\Filter $request, Response $response) {
    
        $request->setParameter('dbname', $this->dataSourceName);
        parent::find($request, $response);
    
    }
}
?>