<?php

namespace Teknisa\SM\Controller;

use Teknisa\SM\Controller\SMCrud;
use Teknisa\Libs\Util\NewCode;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DataSource\DataSet;
use Teknisa\SM\Service\SmPerfilDeAcessoDataSoucerService;
use Teknisa\SM\Util\Utilities;


class SmPerfilDeAcessoDataSoucerController extends SMCrud {
    
    protected $dataSourceName = "descontoAcrescimoPerfil_perfilacesso";
    protected $perfilDeAcessoService;
    protected $newCode;
    
    public function __construct($crudService, $environment, NewCode $newCode,SmPerfilDeAcessoDataSoucerService $perfilDeAcessoService,Utilities $utilities) {
        parent::__construct($crudService, $environment);
        $this->perfilDeAcessoService = $perfilDeAcessoService;
        $this->newCode = $newCode;
        $this->utilities = $utilities;
    }
   /**
    * A função "find" Busca os dados para a tela sem ou com filtro
    */
    public function find(Request\Filter $request, Response $response) {
        $filterData = $request->getFilterCriteria();
        $filtros = array();
        
        if (count($filterData->getConditions()) > 0) {
            foreach ($filterData->getConditions() as $filter) {
                if ($filter["columnName"] == "NRPARCNEGOCIO") {
                    $filtros["NRPARCNEGOCIO"] = $filter["value"];
                }
                else if ($filter["columnName"] == "NRPERFILACESSO") {
                    $filtros["NRPERFILACESSO"] = $filter["value"];
                }
            }
        }
        $resultados = $this->perfilDeAcessoService->find($filtros);
        
        $todosresultados = new DataSet('RESULTADOS', $resultados);
        
        $response->addDataSet($todosresultados);
    }
    
    public function salvarOperacaoAcresOuDescont(Request $request, Response $response) {
        try {
            $row = $request->getRow();
            $nrperfilacesso   = $row["NRPERFILACESSO"];
            $cdservorcadesc   = $row["CDSERVORCADESC"];
            $vrpercentualdesc   = $row["VRPERCENTUALDESC"];
            $idtipodesc   = $row["IDTIPODESC"];
            $cdservorcaacres   = $row["CDSERVORCAACRES"];
            $vrpercentualacres   = $row["VRPERCENTUALACRES"];
            $idtipoacres   = $row["IDTIPOACRES"];
            $row               = $row["ROW"];
            $dataSource    = "fct_percdescperfil_widget";
            $nrorg = $this->environment->getNrOrg();
            $cdoperinclusao = $this->environment->getCdOperador();
            $nrorginclusao   = $nrorg;
            $idativo   = "S";
            $pkFieldNames = ["NRSEQDESCPERF"];
            $validationColumnsDesc = ["CDSERVORCADESC", "NRPERFILACESSO", "IDTIPODESC"];
            $validationColumnsAcres = ["CDSERVORCAACRES", "NRPERFILACESSO", "IDTIPOACRES"];
            $repetido = [];
            
            if($cdservorcadesc && $vrpercentualdesc){
                $inexisteDesc = $this->utilities->validateDuplicatedRecords($row, $dataSource, $pkFieldNames, $validationColumnsDesc);
                if(!$inexisteDesc)
                    array_push($repetido, "D");
            }   
            
            if($cdservorcaacres && $vrpercentualacres){        
                $inexisteAcres = $this->utilities->validateDuplicatedRecords($row, $dataSource, $pkFieldNames, $validationColumnsAcres);
                if(!$inexisteAcres)
                    array_push($repetido, "A");
            }
            
            if(count($repetido) == 0){
                if($cdservorcadesc && $vrpercentualdesc){
                    
                    $nrseqdescperf = $this->newCode->getCodeByTableName("FCT_PERCDESCPERFIL", $nrorg, 1, 10, true);
                    $this->perfilDeAcessoService->salvarOperacaoAcresOuDescont($nrseqdescperf, $nrperfilacesso, $nrorg, $cdservorcadesc, $vrpercentualdesc, $idtipodesc, $cdoperinclusao, $nrorginclusao, $idativo);
                }
                if($cdservorcaacres && $vrpercentualacres){
                    
                    $nrseqdescperf = $this->newCode->getCodeByTableName("FCT_PERCDESCPERFIL", $nrorg, 1, 10, true);
                    $this->perfilDeAcessoService->salvarOperacaoAcresOuDescont($nrseqdescperf, $nrperfilacesso, $nrorg, $cdservorcaacres, $vrpercentualacres, $idtipoacres, $cdoperinclusao, $nrorginclusao, $idativo);
                }
            }
            $resultadosRepetidos = new DataSet('TIPOREPEDITO', $repetido);
        
            $response->addDataSet($resultadosRepetidos);
        }
         catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
}
?>

